local Directions = {
  North = 0,
  East = 1,
  South = 2,
  West = 3,
  NorthEast = 4,
  SouthEast = 5,
  SouthWest = 6,
  NorthWest = 7
}
TargetBot.Creature.attack = function(params, targets, isLooting) -- params {config, creature, danger, priority}
  if player:isWalking() then
    lastWalk = now
  end

  local config = params.config
  local creature = params.creature
  if g_game.getAttackingCreature() ~= creature then
    g_game.attack(creature)
  end

  if not isLooting and type(config) ~= "number" then -- walk only when not looting
    TargetBot.Creature.walk(creature, config, targets)
  end
  
  -- turn to target
  local turnToTarget = storage["TurnToTarget"]
  local pos = g_game.getLocalPlayer():getPosition()
  if turnToTarget and turnToTarget.enabled then
	local tpos = creature:getPosition()
	if pos and tpos then
		local offx = tpos.x - pos.x
		local offy = tpos.y - pos.y
		if offy < 0 and offx <= 0 and math.abs(offx) < math.abs(offy) then
			g_game.turn(Directions.North)
		elseif offy > 0 and offx >= 0 and math.abs(offx) < math.abs(offy) then
			g_game.turn(Directions.South)
		elseif offx < 0 and offy <= 0 and math.abs(offx) > math.abs(offy) then
			g_game.turn(Directions.West)
		elseif offx > 0 and offy >= 0 and math.abs(offx) > math.abs(offy) then
			g_game.turn(Directions.East)
		end
	end
  end
  -- attacks
  local getPvEAttack = storage["Attacks"]["PvE"]
  local aoeCasted = false
  if getPvEAttack and getPvEAttack.enabled then
	local getRadius = getPvEAttack.radius
	if getRadius > 0 and getPvEAttack.area_text:len() > 0 then -- area
		getRadius = math.min(10, getRadius)
		local creatures = g_map.getSpectatorsInRange(pos, false, getRadius, getRadius)
		local targets = 0
		for i=1, #creatures do
			local creaturePtr = creatures[i]
			if creaturePtr:isMonster() then
				targets = targets + 1
			end
		end
		if targets >= getPvEAttack.minTargetRadius then 
			aoeCasted = true
			if TargetBot.sayAttackSpell(getPvEAttack.area_text, getPvEAttack.area_delay) then
				return
			end
		end
	end
	if not aoeCasted and getPvEAttack.target_text:len() > 0 then -- target
		if TargetBot.sayAttackSpell(getPvEAttack.target_text, getPvEAttack.target_delay) then
			return
		end
    end
  end
  local getPvPAttack = storage["Attacks"]["PvP"]
  if getPvPAttack then
	return
  end
end

TargetBot.Creature.walk = function(creature, config, targets)
  local cpos = creature:getPosition()
  local pos = player:getPosition()
  
  local isTrapped = true
  local pos = player:getPosition()
  local dirs = {{-1,1}, {0,1}, {1,1}, {-1, 0}, {1, 0}, {-1, -1}, {0, -1}, {1, -1}}
  for i=1,#dirs do
    local tile = g_map.getTile({x=pos.x-dirs[i][1],y=pos.y-dirs[i][2],z=pos.z})
    if tile and tile:isWalkable(false) then
      isTrapped = false
    end
  end
  
  -- luring
  if TargetBot.canLure() and (config.lure or config.lureCavebot) and not (config.chase and creature:getHealthPercent() < 30) and not isTrapped then
    local monsters = 0
    if targets < config.lureCount then
      if config.lureCavebot then
        return TargetBot.allowCaveBot(200)
      else
        local path = findPath(pos, cpos, 5, {ignoreNonPathable=true, precision=2})
        if path then
          return TargetBot.walkTo(cpos, 10, {marginMin=5, marginMax=6, ignoreNonPathable=true})
        end
      end
    end
  end

  local currentDistance = findPath(pos, cpos, 10, {ignoreCreatures=true, ignoreNonPathable=true, ignoreCost=true})
  if config.chase and (creature:getHealthPercent() < 30 or not config.keepDistance) then
    if #currentDistance > 1 then
      return TargetBot.walkTo(cpos, 10, {ignoreNonPathable=true, precision=1})
    end
  elseif config.keepDistance then
    if #currentDistance ~= config.keepDistanceRange and #currentDistance ~= config.keepDistanceRange + 1 then
      return TargetBot.walkTo(cpos, 10, {ignoreNonPathable=true, marginMin=config.keepDistanceRange, marginMax=config.keepDistanceRange + 1})
    end
  end

  if config.avoidAttacks then
    local diffx = cpos.x - pos.x
    local diffy = cpos.y - pos.y
    local candidates = {}
    if math.abs(diffx) == 1 and diffy == 0 then
      candidates = {{x=pos.x, y=pos.y-1, z=pos.z}, {x=pos.x, y=pos.y+1, z=pos.z}}
    elseif diffx == 0 and math.abs(diffy) == 1 then
      candidates = {{x=pos.x-1, y=pos.y, z=pos.z}, {x=pos.x+1, y=pos.y, z=pos.z}}
    end
    for _, candidate in ipairs(candidates) do
      local tile = g_map.getTile(candidate)
      if tile and tile:isWalkable() then
        return TargetBot.walkTo(candidate, 2, {ignoreNonPathable=true})
      end
    end
  end
end
